@@ -14,10 +14,13 @@
 #    under the License.
 
 import errno
+import json
 import testtools
 
+import six
 import warlock
 
+from glanceclient import exc
 from glanceclient.v2 import images
 from tests import utils
 
@@ -115,7 +118,7 @@ fixtures = {
             },
         ),
     },
-    'v2/images/87b634c1-f893-33c9-28a9-e5673c99239a': {
+    '/v2/images/87b634c1-f893-33c9-28a9-e5673c99239a': {
         'DELETE': (
             {},
             {
@@ -302,12 +305,43 @@ fixtures = {
             {'images': []},
         ),
     },
+    '/v2/images/a2b83adc-888e-11e3-8872-78acc0b951d8': {
+        'GET': (
+            {},
+            {
+                'id': 'a2b83adc-888e-11e3-8872-78acc0b951d8',
+                'name': 'image-location-tests',
+                'locations': [{u'url': u'http://foo.com/',
+                               u'metadata': {u'foo': u'foometa'}},
+                              {u'url': u'http://bar.com/',
+                               u'metadata': {u'bar': u'barmeta'}}],
+            },
+        ),
+        'PATCH': (
+            {},
+            '',
+        )
+    },
 }
 
 
 fake_schema = {
     'name': 'image',
-    'properties': {'id': {}, 'name': {}},
+    'properties': {
+        'id': {},
+        'name': {},
+        'locations': {
+            'type': 'array',
+            'items': {
+                'type': 'object',
+                'properties': {
+                    'metadata': {'type': 'object'},
+                    'url': {'type': 'string'},
+                },
+                'required': ['url', 'metadata'],
+            },
+        },
+    },
     'additionalProperties': {'type': 'string'}
 }
 FakeModel = warlock.model_factory(fake_schema)
@@ -322,50 +356,50 @@ class TestController(testtools.TestCase):
     def test_list_images(self):
         #NOTE(bcwaldon): cast to list since the controller returns a generator
         images = list(self.controller.list())
-        self.assertEqual(images[0].id, '3a4560a1-e585-443e-9b39-553b46ec92d1')
-        self.assertEqual(images[0].name, 'image-1')
-        self.assertEqual(images[1].id, '6f99bf80-2ee6-47cf-acfe-1f1fabb7e810')
-        self.assertEqual(images[1].name, 'image-2')
+        self.assertEqual('3a4560a1-e585-443e-9b39-553b46ec92d1', images[0].id)
+        self.assertEqual('image-1', images[0].name)
+        self.assertEqual('6f99bf80-2ee6-47cf-acfe-1f1fabb7e810', images[1].id)
+        self.assertEqual('image-2', images[1].name)
 
     def test_list_images_paginated(self):
         #NOTE(bcwaldon): cast to list since the controller returns a generator
         images = list(self.controller.list(page_size=1))
-        self.assertEqual(images[0].id, '3a4560a1-e585-443e-9b39-553b46ec92d1')
-        self.assertEqual(images[0].name, 'image-1')
-        self.assertEqual(images[1].id, '6f99bf80-2ee6-47cf-acfe-1f1fabb7e810')
-        self.assertEqual(images[1].name, 'image-2')
+        self.assertEqual('3a4560a1-e585-443e-9b39-553b46ec92d1', images[0].id)
+        self.assertEqual('image-1', images[0].name)
+        self.assertEqual('6f99bf80-2ee6-47cf-acfe-1f1fabb7e810', images[1].id)
+        self.assertEqual('image-2', images[1].name)
 
     def test_list_images_visibility_public(self):
         filters = {'filters': dict([('visibility', 'public')])}
         images = list(self.controller.list(**filters))
-        self.assertEqual(images[0].id, _PUBLIC_ID)
+        self.assertEqual(_PUBLIC_ID, images[0].id)
 
     def test_list_images_visibility_private(self):
         filters = {'filters': dict([('visibility', 'private')])}
         images = list(self.controller.list(**filters))
-        self.assertEqual(images[0].id, _PRIVATE_ID)
+        self.assertEqual(_PRIVATE_ID, images[0].id)
 
     def test_list_images_visibility_shared(self):
         filters = {'filters': dict([('visibility', 'shared')])}
         images = list(self.controller.list(**filters))
-        self.assertEqual(images[0].id, _SHARED_ID)
+        self.assertEqual(_SHARED_ID, images[0].id)
 
     def test_list_images_member_status_rejected(self):
         filters = {'filters': dict([('member_status', 'rejected')])}
         images = list(self.controller.list(**filters))
-        self.assertEqual(images[0].id, _STATUS_REJECTED_ID)
+        self.assertEqual(_STATUS_REJECTED_ID, images[0].id)
 
     def test_list_images_for_owner(self):
         filters = {'filters': dict([('owner', _OWNER_ID)])}
         images = list(self.controller.list(**filters))
-        self.assertEqual(images[0].id, _OWNED_IMAGE_ID)
+        self.assertEqual(_OWNED_IMAGE_ID, images[0].id)
 
     def test_list_images_for_checksum_single_image(self):
         fake_id = '3a4560a1-e585-443e-9b39-553b46ec92d1'
         filters = {'filters': dict([('checksum', _CHKSUM)])}
         images = list(self.controller.list(**filters))
         self.assertEqual(1, len(images))
-        self.assertEqual(images[0].id, '%s' % fake_id)
+        self.assertEqual('%s' % fake_id, images[0].id)
 
     def test_list_images_for_checksum_multiple_images(self):
         fake_id1 = '2a4560b2-e585-443e-9b39-553b46ec92d1'
@@ -373,8 +407,8 @@ class TestController(testtools.TestCase):
         filters = {'filters': dict([('checksum', _CHKSUM1)])}
         images = list(self.controller.list(**filters))
         self.assertEqual(2, len(images))
-        self.assertEqual(images[0].id, '%s' % fake_id1)
-        self.assertEqual(images[1].id, '%s' % fake_id2)
+        self.assertEqual('%s' % fake_id1, images[0].id)
+        self.assertEqual('%s' % fake_id2, images[1].id)
 
     def test_list_images_for_wrong_checksum(self):
         filters = {'filters': dict([('checksum', 'wrong')])}
@@ -384,14 +418,14 @@ class TestController(testtools.TestCase):
     def test_list_images_for_bogus_owner(self):
         filters = {'filters': dict([('owner', _BOGUS_ID)])}
         images = list(self.controller.list(**filters))
-        self.assertEqual(images, [])
+        self.assertEqual([], images)
 
     def test_list_images_for_bunch_of_filters(self):
         filters = {'filters': dict([('owner', _BOGUS_ID),
                                     ('visibility', 'shared'),
                                     ('member_status', 'pending')])}
         images = list(self.controller.list(**filters))
-        self.assertEqual(images[0].id, _EVERYTHING_ID)
+        self.assertEqual(_EVERYTHING_ID, images[0].id)
 
     def test_list_images_filters_encoding(self):
         filters = {"owner": u"ni\xf1o"}
@@ -403,15 +437,17 @@ class TestController(testtools.TestCase):
             #   /v2/images?owner=ni%C3%B1o&limit=20
             # We just want to make sure filters are correctly encoded.
             pass
-
-        self.assertEqual(filters["owner"], "ni\xc3\xb1o")
+        if six.PY2:
+            self.assertEqual("ni\xc3\xb1o", filters["owner"])
+        else:
+            self.assertEqual("ni\xf1o", filters["owner"])
 
     def test_list_images_for_tag_single_image(self):
         img_id = '3a4560a1-e585-443e-9b39-553b46ec92d1'
         filters = {'filters': dict([('tag', [_TAG1])])}
         images = list(self.controller.list(**filters))
         self.assertEqual(1, len(images))
-        self.assertEqual(images[0].id, '%s' % img_id)
+        self.assertEqual('%s' % img_id, images[0].id)
         pass
 
     def test_list_images_for_tag_multiple_images(self):
@@ -420,15 +456,15 @@ class TestController(testtools.TestCase):
         filters = {'filters': dict([('tag', [_TAG2])])}
         images = list(self.controller.list(**filters))
         self.assertEqual(2, len(images))
-        self.assertEqual(images[0].id, '%s' % img_id1)
-        self.assertEqual(images[1].id, '%s' % img_id2)
+        self.assertEqual('%s' % img_id1, images[0].id)
+        self.assertEqual('%s' % img_id2, images[1].id)
 
     def test_list_images_for_multi_tags(self):
         img_id1 = '2a4560b2-e585-443e-9b39-553b46ec92d1'
         filters = {'filters': dict([('tag', [_TAG1, _TAG2])])}
         images = list(self.controller.list(**filters))
         self.assertEqual(1, len(images))
-        self.assertEqual(images[0].id, '%s' % img_id1)
+        self.assertEqual('%s' % img_id1, images[0].id)
 
     def test_list_images_for_non_existent_tag(self):
         filters = {'filters': dict([('tag', ['fake'])])}
@@ -437,16 +473,16 @@ class TestController(testtools.TestCase):
 
     def test_get_image(self):
         image = self.controller.get('3a4560a1-e585-443e-9b39-553b46ec92d1')
-        self.assertEqual(image.id, '3a4560a1-e585-443e-9b39-553b46ec92d1')
-        self.assertEqual(image.name, 'image-1')
+        self.assertEqual('3a4560a1-e585-443e-9b39-553b46ec92d1', image.id)
+        self.assertEqual('image-1', image.name)
 
     def test_create_image(self):
         properties = {
             'name': 'image-1'
         }
         image = self.controller.create(**properties)
-        self.assertEqual(image.id, '3a4560a1-e585-443e-9b39-553b46ec92d1')
-        self.assertEqual(image.name, 'image-1')
+        self.assertEqual('3a4560a1-e585-443e-9b39-553b46ec92d1', image.id)
+        self.assertEqual('image-1', image.name)
 
     def test_create_bad_additionalProperty_type(self):
         properties = {
@@ -460,10 +496,10 @@ class TestController(testtools.TestCase):
         self.controller.delete('87b634c1-f893-33c9-28a9-e5673c99239a')
         expect = [
             ('DELETE',
-                'v2/images/87b634c1-f893-33c9-28a9-e5673c99239a',
+                '/v2/images/87b634c1-f893-33c9-28a9-e5673c99239a',
                 {},
                 None)]
-        self.assertEqual(self.api.calls, expect)
+        self.assertEqual(expect, self.api.calls)
 
     def test_data_upload(self):
         image_data = 'CCC'
@@ -472,23 +508,32 @@ class TestController(testtools.TestCase):
         expect = [('PUT', '/v2/images/%s/file' % image_id,
                   {'Content-Type': 'application/octet-stream'},
                   image_data)]
-        self.assertEqual(self.api.calls, expect)
+        self.assertEqual(expect, self.api.calls)
+
+    def test_data_upload_w_size(self):
+        image_data = 'CCC'
+        image_id = '606b0e88-7c5a-4d54-b5bb-046105d4de6f'
+        self.controller.upload(image_id, image_data, image_size=3)
+        expect = [('PUT', '/v2/images/%s/file' % image_id,
+                  {'Content-Type': 'application/octet-stream'},
+                  image_data, 3)]
+        self.assertEqual(expect, self.api.calls)
 
     def test_data_without_checksum(self):
         body = self.controller.data('5cc4bebc-db27-11e1-a1eb-080027cbe205',
                                     do_checksum=False)
         body = ''.join([b for b in body])
-        self.assertEqual(body, 'A')
+        self.assertEqual('A', body)
 
         body = self.controller.data('5cc4bebc-db27-11e1-a1eb-080027cbe205')
         body = ''.join([b for b in body])
-        self.assertEqual(body, 'A')
+        self.assertEqual('A', body)
 
     def test_data_with_wrong_checksum(self):
         body = self.controller.data('66fb18d6-db27-11e1-a1eb-080027cbe205',
                                     do_checksum=False)
         body = ''.join([b for b in body])
-        self.assertEqual(body, 'BB')
+        self.assertEqual('BB', body)
 
         body = self.controller.data('66fb18d6-db27-11e1-a1eb-080027cbe205')
         try:
@@ -503,11 +548,11 @@ class TestController(testtools.TestCase):
         body = self.controller.data('1b1c6366-dd57-11e1-af0f-02163e68b1d8',
                                     do_checksum=False)
         body = ''.join([b for b in body])
-        self.assertEqual(body, 'CCC')
+        self.assertEqual('CCC', body)
 
         body = self.controller.data('1b1c6366-dd57-11e1-af0f-02163e68b1d8')
         body = ''.join([b for b in body])
-        self.assertEqual(body, 'CCC')
+        self.assertEqual('CCC', body)
 
     def test_update_replace_prop(self):
         image_id = '3a4560a1-e585-443e-9b39-553b46ec92d1'
@@ -522,11 +567,11 @@ class TestController(testtools.TestCase):
             ('PATCH', '/v2/images/%s' % image_id, expect_hdrs, expect_body),
             ('GET', '/v2/images/%s' % image_id, {}, None),
         ]
-        self.assertEqual(self.api.calls, expect)
-        self.assertEqual(image.id, image_id)
+        self.assertEqual(expect, self.api.calls)
+        self.assertEqual(image_id, image.id)
         #NOTE(bcwaldon): due to limitations of our fake api framework, the name
         # will not actually change - yet in real life it will...
-        self.assertEqual(image.name, 'image-1')
+        self.assertEqual('image-1', image.name)
 
     def test_update_add_prop(self):
         image_id = '3a4560a1-e585-443e-9b39-553b46ec92d1'
@@ -541,11 +586,11 @@ class TestController(testtools.TestCase):
             ('PATCH', '/v2/images/%s' % image_id, expect_hdrs, expect_body),
             ('GET', '/v2/images/%s' % image_id, {}, None),
         ]
-        self.assertEqual(self.api.calls, expect)
-        self.assertEqual(image.id, image_id)
+        self.assertEqual(expect, self.api.calls)
+        self.assertEqual(image_id, image.id)
         #NOTE(bcwaldon): due to limitations of our fake api framework, the name
         # will not actually change - yet in real life it will...
-        self.assertEqual(image.name, 'image-1')
+        self.assertEqual('image-1', image.name)
 
     def test_update_remove_prop(self):
         image_id = 'e7e59ff6-fa2e-4075-87d3-1a1398a07dc3'
@@ -560,11 +605,11 @@ class TestController(testtools.TestCase):
             ('PATCH', '/v2/images/%s' % image_id, expect_hdrs, expect_body),
             ('GET', '/v2/images/%s' % image_id, {}, None),
         ]
-        self.assertEqual(self.api.calls, expect)
-        self.assertEqual(image.id, image_id)
+        self.assertEqual(expect, self.api.calls)
+        self.assertEqual(image_id, image.id)
         #NOTE(bcwaldon): due to limitations of our fake api framework, the name
         # will not actually change - yet in real life it will...
-        self.assertEqual(image.name, 'image-3')
+        self.assertEqual('image-3', image.name)
 
     def test_update_replace_remove_same_prop(self):
         image_id = 'e7e59ff6-fa2e-4075-87d3-1a1398a07dc3'
@@ -582,11 +627,11 @@ class TestController(testtools.TestCase):
             ('PATCH', '/v2/images/%s' % image_id, expect_hdrs, expect_body),
             ('GET', '/v2/images/%s' % image_id, {}, None),
         ]
-        self.assertEqual(self.api.calls, expect)
-        self.assertEqual(image.id, image_id)
+        self.assertEqual(expect, self.api.calls)
+        self.assertEqual(image_id, image.id)
         #NOTE(bcwaldon): due to limitations of our fake api framework, the name
         # will not actually change - yet in real life it will...
-        self.assertEqual(image.name, 'image-3')
+        self.assertEqual('image-3', image.name)
 
     def test_update_add_remove_same_prop(self):
         image_id = 'e7e59ff6-fa2e-4075-87d3-1a1398a07dc3'
@@ -603,14 +648,116 @@ class TestController(testtools.TestCase):
             ('PATCH', '/v2/images/%s' % image_id, expect_hdrs, expect_body),
             ('GET', '/v2/images/%s' % image_id, {}, None),
         ]
-        self.assertEqual(self.api.calls, expect)
-        self.assertEqual(image.id, image_id)
+        self.assertEqual(expect, self.api.calls)
+        self.assertEqual(image_id, image.id)
         #NOTE(bcwaldon): due to limitations of our fake api framework, the name
         # will not actually change - yet in real life it will...
-        self.assertEqual(image.name, 'image-3')
+        self.assertEqual('image-3', image.name)
 
     def test_update_bad_additionalProperty_type(self):
         image_id = 'e7e59ff6-fa2e-4075-87d3-1a1398a07dc3'
         params = {'name': 'pong', 'bad_prop': False}
         with testtools.ExpectedException(TypeError):
             self.controller.update(image_id, **params)
+
+    def test_location_ops_when_server_disabled_location_ops(self):
+        # Location operations should not be allowed if server has not
+        # enabled location related operations
+        image_id = '3a4560a1-e585-443e-9b39-553b46ec92d1'
+        estr = 'The administrator has disabled API access to image locations'
+        url = 'http://bar.com/'
+        meta = {'bar': 'barmeta'}
+
+        e = self.assertRaises(exc.HTTPBadRequest,
+                              self.controller.add_location,
+                              image_id, url, meta)
+        self.assertTrue(estr in str(e))
+
+        e = self.assertRaises(exc.HTTPBadRequest,
+                              self.controller.delete_locations,
+                              image_id, set([url]))
+        self.assertTrue(estr in str(e))
+
+        e = self.assertRaises(exc.HTTPBadRequest,
+                              self.controller.update_location,
+                              image_id, url, meta)
+        self.assertTrue(estr in str(e))
+
+    def _empty_get(self, image_id):
+        return ('GET', '/v2/images/%s' % image_id, {}, None)
+
+    def _patch_req(self, image_id, patch_body):
+        c_type = 'application/openstack-images-v2.1-json-patch'
+        return ('PATCH',
+                '/v2/images/%s' % image_id,
+                {'Content-Type': c_type},
+                json.dumps(patch_body))
+
+    def test_add_location(self):
+        image_id = 'a2b83adc-888e-11e3-8872-78acc0b951d8'
+        new_loc = {'url': 'http://spam.com/', 'metadata': {'spam': 'ham'}}
+        add_patch = {'path': '/locations/-', 'value': new_loc, 'op': 'add'}
+        image = self.controller.add_location(image_id, **new_loc)
+        self.assertEqual(self.api.calls, [
+            self._empty_get(image_id),
+            self._patch_req(image_id, [add_patch]),
+            self._empty_get(image_id)
+        ])
+
+    def test_add_duplicate_location(self):
+        image_id = 'a2b83adc-888e-11e3-8872-78acc0b951d8'
+        new_loc = {'url': 'http://foo.com/', 'metadata': {'foo': 'newfoo'}}
+        err_str = 'A location entry at %s already exists' % new_loc['url']
+
+        err = self.assertRaises(exc.HTTPConflict,
+                                self.controller.add_location,
+                                image_id, **new_loc)
+        self.assertIn(err_str, str(err))
+
+    def test_remove_location(self):
+        image_id = 'a2b83adc-888e-11e3-8872-78acc0b951d8'
+        url_set = set(['http://foo.com/', 'http://bar.com/'])
+        del_patches = [{'path': '/locations/1', 'op': 'remove'},
+                       {'path': '/locations/0', 'op': 'remove'}]
+        image = self.controller.delete_locations(image_id, url_set)
+        self.assertEqual(self.api.calls, [
+            self._empty_get(image_id),
+            self._patch_req(image_id, del_patches)
+        ])
+
+    def test_remove_missing_location(self):
+        image_id = 'a2b83adc-888e-11e3-8872-78acc0b951d8'
+        url_set = set(['http://spam.ham/'])
+        err_str = 'Unknown URL(s): %s' % list(url_set)
+
+        err = self.assertRaises(exc.HTTPNotFound,
+                                self.controller.delete_locations,
+                                image_id, url_set)
+        self.assertTrue(err_str in str(err))
+
+    def test_update_location(self):
+        image_id = 'a2b83adc-888e-11e3-8872-78acc0b951d8'
+        new_loc = {'url': 'http://foo.com/', 'metadata': {'spam': 'ham'}}
+        fixture_idx = '/v2/images/%s' % (image_id)
+        orig_locations = fixtures[fixture_idx]['GET'][1]['locations']
+        loc_map = dict([(l['url'], l) for l in orig_locations])
+        loc_map[new_loc['url']] = new_loc
+        mod_patch = [{'path': '/locations', 'op': 'replace',
+                      'value': []},
+                     {'path': '/locations', 'op': 'replace',
+                      'value': list(loc_map.values())}]
+        image = self.controller.update_location(image_id, **new_loc)
+        self.assertEqual(self.api.calls, [
+            self._empty_get(image_id),
+            self._patch_req(image_id, mod_patch),
+            self._empty_get(image_id)
+        ])
+
+    def test_update_missing_location(self):
+        image_id = 'a2b83adc-888e-11e3-8872-78acc0b951d8'
+        new_loc = {'url': 'http://spam.com/', 'metadata': {'spam': 'ham'}}
+        err_str = 'Unknown URL: %s' % new_loc['url']
+        err = self.assertRaises(exc.HTTPNotFound,
+                                self.controller.update_location,
+                                image_id, **new_loc)
+        self.assertTrue(err_str in str(err))
